#! /usr/bin/env python
# Copyright (C) 2010 by Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Script to generate buildout configuration files."""

import os

import helpers


def generate_buildout_cfg(distro, dest_dir='.', root_dir='.'):
    """Generate a buildout configuration file.

    :param distro: a dict of projects, doc_sets and downloads.
       See bazaar_releases for examples.
    :param dest_dir: the destination directory
    :return: the name of the generated buildout file
    """
    cfg_file = 'buildout-%s.cfg' % (distro['name'],)
    print "Generating %s in %s" % (cfg_file, dest_dir)
    cfg_path = os.path.join(dest_dir, cfg_file)
    this_dir = os.path.dirname(__file__)
    template_path = os.path.join(this_dir, "templates", "buildout",
        "buildout.cfg")
    context = helpers.context_for_distro(distro, root_dir)
    context['parts'] = _get_parts(distro)
    helpers.jinja_compile(template_path, context, cfg_path)
    return cfg_path


def _get_parts(distro):
    """Get the list of parts in a distro."""
    downloads = [d.part_name for d in distro['downloads']
        + distro['tortoise_overlays']]
    projects = [p.lp_project for p in distro['projects']]
    plugins = [p.lp_project for p in distro['plugins'] if not p.core]
    return downloads + projects + plugins


if __name__ == '__main__':
    import sys
    if len(sys.argv[1:]) == 0:
        print "usage: buildoutgen distro-name [root-dir]"
        sys.exit(1)
    distro = helpers.find_distros([sys.argv[1]])[0]
    try:
        root_dir = sys.argv[2]
    except IndexError:
        root_dir = '.'
    generate_buildout_cfg(distro, root_dir=root_dir)
