; Script for Inno Setup installer
; bzr-windows-installer project version

[Setup]
AppName=Bazaar
AppVerName=Bazaar {{ version }}
OutputBaseFilename=bzr-{{ version }}-setup
DefaultDirName={pf}\Bazaar
DefaultGroupName=Bazaar
SolidCompression=yes

OutputDir="..\"
SourceDir="..\..\win32_bzr.exe"

SetupIconFile="..\bzr.ico"
WizardSmallImageFile="bzr-48.bmp"
WizardImageFile="bzr-164x314.bmp"
InfoBeforeFile="..\tools\win32\info.txt"

VersionInfoCompany="Canonical Ltd."
VersionInfoCopyright="Canonical Ltd., 2005-2010"
VersionInfoDescription="Windows installer for Bazaar"
VersionInfoVersion="{{ VersionInfoVersion }}"

AppComments="Bazaar: adaptive version control system"
AppPublisher="Canonical Ltd."
AppPublisherURL="http://www.bazaar.canonical.com"
AppSupportURL="http://wiki.bazaar.canonical.com/BzrSupport"
AppUpdatesURL="http://wiki.bazaar.canonical.com/WindowsDownloads"
AppVersion={{ version }}

ChangesEnvironment=yes
; MARKH: PrivilegesRequired=none means it can't be installed by a non-admin
; user - but sadly we still need admin - eg, tortoise overlays, installing
; into "\Program Files", installing COM objects etc all must be done by an
; admin.
PrivilegesRequired=admin

TimeStampsInUTC=yes
; The modified time of files can't be the default translated localtime, as
; Python stores the timestamp in bytecode files and will recompile them at
; runtime if the installer changes the source files' modified time.


[Files]
{% if tbzr -%}
; Tortoise files - these are at the top as we use 'ExtractTemporaryFile' on
; the TortoiseOverlays MSI, and inno documents such files should be at the
; start for best performance.
Source: "{{ tbzr_msi32 }}"; Flags: dontcopy ignoreversion ; Components: tortoise
Source: "{{ tbzr_msi64 }}"; Flags: dontcopy ignoreversion ; Components: tortoise
Source: "tbzrcache.exe"; DestDir: "{app}"; Flags: ignoreversion restartreplace uninsrestartdelete; Components: tortoise
Source: "tbzrcachew.exe"; DestDir: "{app}"; Flags: ignoreversion restartreplace uninsrestartdelete; Components: tortoise
Source: "tbzrcommand.exe"; DestDir: "{app}"; Flags: ignoreversion restartreplace uninsrestartdelete; Components: tortoise
Source: "tbzrcommandw.exe"; DestDir: "{app}"; Flags: ignoreversion restartreplace uninsrestartdelete; Components: tortoise
Source: "tbzrtrace.exe"; DestDir: "{app}"; Flags: ignoreversion restartreplace uninsrestartdelete; Components: debug
; Note 'regserver' here appears to run regsvr32 without elevation, which
; is no good for us - so we have a [run] entry below.
Source: "tbzrshellext_x86.dll"; DestDir: "{app}"; Flags: ignoreversion restartreplace uninsrestartdelete; Components: tortoise
Source: "tbzrshellext_x64.dll"; DestDir: "{app}"; Flags: ignoreversion restartreplace uninsrestartdelete; Components: tortoise; Check: IsWin64;
Source: "{{ tbzr }}\\doc\\*.html"; DestDir: "{app}\\doc\\tbzr"; Flags: ignoreversion; Components: tortoise
Source: "{{ tbzr }}\\locale\\*.mo"; DestDir: "{app}\\locale"; Flags: createallsubdirs ignoreversion recursesubdirs; Components: tortoise
{%- endif %}

; We can't say '*.*' due to optional components.
Source: "*.bat"; DestDir: "{app}"; Flags: ignoreversion restartreplace uninsrestartdelete;
Source: "*.url"; DestDir: "{app}"; Flags: ignoreversion restartreplace uninsrestartdelete;
Source: "msvc*.dll"; DestDir: "{app}"; Flags: ignoreversion restartreplace uninsrestartdelete;
Source: "bz*.exe"; DestDir: "{app}"; Flags: ignoreversion restartreplace uninsrestartdelete;
Source: "Python*.dll"; DestDir: "{app}"; Flags: ignoreversion restartreplace uninsrestartdelete;
Source: "lib\*.*"; DestDir: "{app}\lib"; Flags: createallsubdirs ignoreversion recursesubdirs restartreplace uninsrestartdelete;

{% for docset in docsets if not docset.localization -%}
Source: "bzr-{{ docset.id }}.chm"; DestDir: "{app}"; Components: doc\{{ docset.id }}; Flags: ignoreversion restartreplace uninsrestartdelete;
{% endfor %}
{% for docset in docsets if docset.localization -%}
Source: "bzr-{{ docset.id }}.chm"; DestDir: "{app}"; Components: l10ndoc\{{ docset.id }}; Flags: ignoreversion restartreplace uninsrestartdelete;
{% endfor %}

{% for plugin in plugins -%}
{% if plugin.id == 'explorer' -%}
    {% set components = 'explorer' -%}
{% elif plugin.id == 'rebase' -%}
    {% set components = 'plugins\\rebase plugins\\svn' -%}
{% else -%}
    {% set components = 'plugins\\' + plugin.id -%}
{%- endif %}
Source: "plugins\{{ plugin.id }}\*.*"; DestDir: "{app}\plugins\{{ plugin.id }}"; Components: {{ components }}; Flags: createallsubdirs ignoreversion recursesubdirs restartreplace uninsrestartdelete;
{%- endfor %}

; imageformats plugins for PyQt4
Source: "imageformats\\*.*"; DestDir: "{app}\\imageformats"; Flags: createallsubdirs ignoreversion recursesubdirs restartreplace uninsrestartdelete;

[Types]
Name: "typical"; Description: "A typical installation"
Name: "full"; Description: "Full Installation (typical installation plus test utilities)"
Name: "compact"; Description: "Compact installation"
Name: "custom"; Description: "Custom installation"; Flags: iscustom


[Components]
Name: "main"; Description: "Main Files"; Types: full typical compact custom; Flags: fixed
{% for appl in applications -%}
Name: "{{ appl.id }}"; Description: "{{ appl.description }}"; Types: full typical custom;
{%- endfor %}
{% if tbzr -%}
Name: "tortoise"; Description: "Windows Shell Extensions (TortoiseBZR)"; Types: full custom;
Name: "debug"; Description: "Test, diagnostic and debugging utilities"; Types: full custom;
{%- endif %}

Name: "doc"; Description: "Documentation"; Types: full typical custom;
{% for docset in docsets if not docset.localization -%}
Name: "doc\{{ docset.id }}"; Description: "{{ docset.description }}"; Types: full typical custom;
{% endfor %}
{% for docset in docsets if docset.localization -%}
{% if loop.first -%}
Name: "l10ndoc"; Description: "Localized Documentation"; Types: full custom;
{% endif -%}
Name: "l10ndoc\{{ docset.id }}"; Description: "{{ docset.description }}"; Types: full custom;
{% endfor %}

{# Maybe we ought to make this a group/members nested for loop pair? #}
Name: "plugins"; Description: "Plugins"; Types: full typical custom;
{% for plugin in plugins if plugin.display_in_installer -%}
Name: "plugins\{{ plugin.id }}"; Description: "{{ plugin.description }}"; Types: full typical custom;
{% endfor %}


[Dirs]
Name: "{userappdata}\bazaar\2.0"
Name: "{app}\plugins"; Flags: uninsalwaysuninstall


[Icons]
{% for appl in applications -%}
Name: "{group}\{{ appl.name }}"; Filename: "{app}\bzr.exe"; WorkingDir: {sd}\; Parameters: "{{ appl.params }}"; Components: {{ appl.id }};
Name: "{commondesktop}\{{ appl.name }}"; Filename: "{app}\bzr.exe"; Parameters: "{{ appl.params }}"; WorkingDir: {sd}\; Tasks: desktopicon
Name: "{userappdata}\Microsoft\Internet Explorer\Quick Launch\{{ appl.name }}"; Filename: "{app}\bzr.exe"; Parameters: "{{ appl.params }}"; WorkingDir: {sd}\; Tasks: quicklaunchicon
{%- endfor %}

{% for docset in docsets if not docset.localization -%}
Name: "{group}\{{ docset.description }}"; Filename: "{app}\bzr-{{ docset.id }}.chm"; Components: doc\{{ docset.id }};
{% endfor %}
{% for docset in docsets if docset.localization -%}
Name: "{group}\{{ docset.description }}"; Filename: "{app}\bzr-{{ docset.id }}.chm"; Components: l10ndoc\{{ docset.id }};
{% endfor %}
{% if tbzr -%}
Name: "{group}\TortoiseBZR documentation"; Filename: "{app}\doc\tbzr\index.html"; Comment: "Launch TortoiseBZR documentation"; Components: tortoise;
{%- endif %}

Name: "{group}\Bazaar Home Page"; Filename: "{app}\bazaar.url"; Comment: "http://www.bazaar.canonical.com";
Name: "{group}\bzr command line tool"; Filename: "{cmd}"; Parameters: "/K start_bzr.bat"; WorkingDir: "{app}"; IconFilename: "{app}\bzr.exe"; Comment: "Open command prompt and run bzr";
; NOTE: Intent is to change the log file location - the line below will need to change to reflect that.
Name: "{group}\Bazaar Log File"; Filename: "notepad.exe"; Parameters: "{userdocs}\.bzr.log"; Comment: "Launch notepad to view the Bazaar log file";


[Tasks]
Name: Path; Description: "Add {app} directory to PATH environment variable"
{% for appl in applications -%}
{# TODO: change the task names to include appl.id #}
Name: "desktopicon"; Description: "Create {{ appl.name }} &desktop icon"; GroupDescription: "{cm:AdditionalIcons}"; Components: {{ appl.id }};
Name: "quicklaunchicon"; Description: "Create {{ appl.name }} &Quick Launch icon"; GroupDescription: "{cm:AdditionalIcons}";  Components: {{ appl.id }}; Flags: unchecked
{%- endfor %}
{% if tbzr -%}
Name: TBZRReadme; Description: "View the TortoiseBZR Readme"; Components: tortoise; Flags: unchecked
{%- endif %}


[Registry]
Root: HKLM; Subkey: "SOFTWARE\Bazaar"; Flags: noerror uninsdeletekey
Root: HKLM; Subkey: "SOFTWARE\Bazaar"; ValueName: "InstallPath"; ValueType: string; ValueData: "{app}"; Flags: noerror
; Don't write stuff that can be implied
;Root: HKLM; Subkey: "SOFTWARE\Bazaar"; ValueName: "BzrlibPath"; ValueType: string; ValueData: "{app}\lib\library.zip\bzrlib"; Flags: noerror
;Root: HKLM; Subkey: "SOFTWARE\Bazaar"; ValueName: "PluginsPath"; ValueType: string; ValueData: "{app}\plugins"; Flags: noerror
;Root: HKLM; Subkey: "SOFTWARE\Bazaar"; ValueName: "PythonPath"; ValueType: string; ValueData: "{app}\lib\library.zip"; Flags: noerror
;Root: HKLM; Subkey: "SOFTWARE\Bazaar"; ValueName: "Version"; ValueType: string; ValueData: "{{ version }}"; Flags: noerror


[Run]
Filename: "{app}\bzr_postinstall.exe"; Parameters: "--start-bzr"; Flags: skipifdoesntexist runhidden;
Filename: "{app}\bzr_postinstall.exe"; Parameters: "--add-path"; Tasks: Path; Flags: skipifdoesntexist skipifsilent runhidden;
{% if tbzr -%}
Filename: "regsvr32.exe"; Parameters: "/s /i: /n tbzrshellext_x86.dll"; WorkingDir: "{app}"; Components: tortoise; Description: "Registering Tortoise"; StatusMsg: "Registering Tortoise"
Filename: "regsvr32.exe"; Parameters: "/s /i: /n tbzrshellext_x64.dll"; WorkingDir: "{app}"; Components: tortoise; Description: "Registering Tortoise"; StatusMsg: "Registering Tortoise"; Check: IsWin64
Filename: "{app}\doc\tbzr\index.html"; Tasks: TBZRReadme; Flags: shellexec
{%- endif %}


[UninstallRun]
Filename: "{app}\bzr_postinstall.exe"; Parameters: "--delete-path --delete-shell-menu --silent"; Flags: skipifdoesntexist runhidden;
{% if tbzr -%}
Filename: "regsvr32.exe"; Parameters: "/u /s /i: tbzrshellext_x86.dll"; WorkingDir: "{app}"; Components: tortoise; StatusMsg: "Unregistering Tortoise"; Flags: skipifdoesntexist
Filename: "regsvr32.exe"; Parameters: "/u /s /i: tbzrshellext_x64.dll"; WorkingDir: "{app}"; Components: tortoise; StatusMsg: "Unregistering Tortoise"; Flags: skipifdoesntexist; Check: IsWin64
{%- endif %}


[Code]
const
  SHCNF_IDLIST = $0000;
  SHCNE_ASSOCCHANGED = $08000000;
  WM_QUIT = 18;
  MOVEFILE_DELAY_UNTIL_REBOOT    = 4;

procedure SHChangeNotify(wEventId, uFlags, dwItem1, dwItem2: Integer);
 external 'SHChangeNotify@shell32.dll stdcall';

function MoveFileEx(lpExistingFileName, lpNewFileName: String; dwFlags: Cardinal): Integer;
 external 'MoveFileExA@kernel32.dll stdcall';

procedure DeleteFileNowOrLater(filename: string);
var
  rc : Integer;
begin
    if FileExists(filename) and not DeleteFile(filename) then
        // can't work out to pass NULL to the API, but an empty string
        // seems to work OK.
        MoveFileEx(filename, '', MOVEFILE_DELAY_UNTIL_REBOOT);
end;

procedure ShutdownTBZR;
var
    hwnd: HWND;
begin
{% if tbzr -%}
    // ask the cache process to shut-down.
    hwnd := FindWindowByClassName('TBZRCache_Taskbar');
    if hwnd <> 0 then
        PostMessage(hwnd, WM_QUIT, 1, 0);
{% else -%}
    Exit;  // No TSVN set - exit this procedure.
{%- endif %}
end;

procedure CurStepChanged(CurStep: TSetupStep);
var
    S, tovmsi, fqtovmsi, params: String;
    ErrorCode: Integer;
begin
    if CurStep=ssInstall then begin
        ShutdownTBZR;
        // In case the user hasn't uninstalled the old version before
        // upgrading, we unregister and delete some obsolete files
        // (regsvr32 remains silent even if the file doesn't exist)
        Exec('regsvr32.exe', '/s /u "' + ExpandConstant('{app}\tbzr.dll') + '"',
             '', SW_HIDE, ewWaitUntilTerminated, ErrorCode);
        DeleteFileNowOrLater(ExpandConstant('{app}\tbzr.dll'));
        DeleteFileNowOrLater(ExpandConstant('{app}\tbzrtest.exe'));
        DeleteFileNowOrLater(ExpandConstant('{app}\tbzr_tracer.exe'));
    end;

    if CurStep=ssPostInstall then begin
        // a couple of post-install tasks
        if IsComponentSelected('tortoise') then begin
            // Need to execute:
            // msiexec /i TortoiseOverlays-1.X.X.XXXX-win32.msi /qn /norestart
            // 64bit notes:
            // We are still primarily a 32bit application - the only 64bit binary is the
            // shell extension, but even then, we need to install the 32bit version too.
            // Thus, we keep tortoise in 32bit "install mode" - meaning we are installed
            // to "\Program Files (x86)".  We don't bother trying to install our single
            // 64bit DLL into "\Program Files" - we use a different DLL name for 32 and
            // 64 bit versions, so nothing will conflict.
            // Note however that on a 64bit OS, we only need the 64bit TortoiseOverlays -
            // the 32bit apps using shell extensions still work fine with that.
            {% if tbzr -%}
            if IsWin64 then
                tovmsi := '{{ tbzr_msi64_basename }}'
            else
                tovmsi := '{{ tbzr_msi32_basename }}'
            {% else -%}
            {# XXX: Is this else clause necessary? I suspect not. (IGC 10/Mar/2010) #}
            tovmsi := '';
            {%- endif %}
            ExtractTemporaryFile(tovmsi);
            fqtovmsi := AddBackslash(ExpandConstant('{tmp}')) + tovmsi;
            params := '/i "' + fqtovmsi + '" /qn /norestart';
            if not ShellExec('', 'msiexec.exe', params, '', SW_HIDE,
                             ewWaitUntilTerminated, ErrorCode) then
                MsgBox('Failed to install TortoiseOverlays: ' + SysErrorMessage(ErrorCode),
                        mbInformation, MB_OK);
            // Ideally we could be bzr_postinstall.exe this way too, but
            // its needed at uninstall time.
        end;
        // cause explorer to re-fetch handlers.
        SHChangeNotify(SHCNE_ASSOCCHANGED, SHCNF_IDLIST, 0, 0);
    end;
end;


function InitializeUninstall(): Boolean;
begin
    ShutdownTBZR;
    result := True;
end;
